package com.jiuqi.syntax;

import java.util.HashMap;
import java.io.PrintStream;

public class NiporlanItemFactory
{
    public PrintStream logger;
    private long id;
    private static NiporlanItemFactory globalFactory;
    private static boolean useGlobalFactory;
    protected static HashMap OP_NIS;
    protected HashMap FUNC_NIS;
    protected static NiporlanItem BOOL_TRUE_NI;
    protected static NiporlanItem BOOL_FALSE_NI;
    protected static NiporlanItem BOOL_NULL_NI;
    public int FOUND_COUNT;
    public int MISSED_COUNT;
    public int NULL_COUNT;
    protected HashMap[] dataCellCache;
    protected static int PAGE_COUNT;
    
    static {
        NiporlanItemFactory.globalFactory = new NiporlanItemFactory();
        NiporlanItemFactory.useGlobalFactory = true;
        NiporlanItemFactory.OP_NIS = new HashMap();
        for (int i = 1; i <= 20; ++i) {
            NiporlanItemFactory.OP_NIS.put(new Integer(i), new NiporlanItem(i));
        }
        CommonData cd = new CommonData();
        cd.type = 5;
        cd.isNull = false;
        cd.cdBool = true;
        NiporlanItemFactory.BOOL_TRUE_NI = new NiporlanItem(cd);
        cd = new CommonData();
        cd.type = 5;
        cd.isNull = false;
        cd.cdBool = false;
        NiporlanItemFactory.BOOL_FALSE_NI = new NiporlanItem(cd);
        cd = new CommonData();
        cd.type = 5;
        cd.isNull = true;
        NiporlanItemFactory.BOOL_NULL_NI = new NiporlanItem(cd);
        NiporlanItemFactory.PAGE_COUNT = 2000;
    }
    
    public static boolean getUseGlobalFactory() {
        return NiporlanItemFactory.useGlobalFactory;
    }
    
    public static void setUseGlobalFactory(final boolean value) {
        NiporlanItemFactory.useGlobalFactory = value;
    }
    
    public static NiporlanItemFactory getFactory() {
        if (NiporlanItemFactory.useGlobalFactory) {
            return NiporlanItemFactory.globalFactory;
        }
        return new NiporlanItemFactory();
    }
    
    public NiporlanItemFactory() {
        this.logger = null;
        this.id = System.currentTimeMillis();
        this.FUNC_NIS = new HashMap();
        this.FOUND_COUNT = 0;
        this.MISSED_COUNT = 0;
        this.NULL_COUNT = 0;
        this.dataCellCache = null;
        if (this.logger != null) {
            this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "Crate NO." + this.id + " NIFactory.");
        }
    }
    
    public NiporlanItem getNiporlanItem(final int opCode) {
        NiporlanItem ni = null;
        synchronized (NiporlanItemFactory.OP_NIS) {
            ni = NiporlanItemFactory.OP_NIS.get(new Integer(opCode));
            if (ni == null) {
                ni = new NiporlanItem(opCode);
                NiporlanItemFactory.OP_NIS.put(new Integer(opCode), ni);
                if (this.logger != null) {
                    this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "添加新的操作符(" + opCode + ")共享NiporlanItem ");
                }
            }
            monitorexit(NiporlanItemFactory.OP_NIS);
        }
        return ni;
    }
    
    public NiporlanItem getNiporlanItem(final String funcName) {
        if (funcName == null) {
            return null;
        }
        NiporlanItem ni = null;
        synchronized (this.FUNC_NIS) {
            ni = this.FUNC_NIS.get(funcName);
            if (ni == null) {
                ni = new NiporlanItem(funcName);
                this.FUNC_NIS.put(funcName, ni);
                if (this.logger != null) {
                    this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "添加新的函数(" + funcName + ")共享NiporlanItem ");
                }
            }
            monitorexit(this.FUNC_NIS);
        }
        return ni;
    }
    
    public NiporlanItem getNiporlanItem(final CommonData commonData) {
        if (commonData != null && commonData.type == 5) {
            if (commonData.isNull) {
                return NiporlanItemFactory.BOOL_NULL_NI;
            }
            if (commonData.cdBool) {
                return NiporlanItemFactory.BOOL_TRUE_NI;
            }
            if (!commonData.cdBool) {
                return NiporlanItemFactory.BOOL_FALSE_NI;
            }
        }
        return new NiporlanItem(commonData);
    }
    
    public NiporlanItem getNiporlanItem(final DataCell dataCell) {
        if (this.dataCellCache == null) {
            this.dataCellCache = new HashMap[NiporlanItemFactory.PAGE_COUNT];
            for (int i = 0; i < this.dataCellCache.length; ++i) {
                this.dataCellCache[i] = new HashMap();
            }
            if (this.logger != null) {
                this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "DataCell缓冲池初始化 " + System.currentTimeMillis());
            }
        }
        if (dataCell == null) {
            if (this.logger != null) {
                this.logger.println(" DataCell is null");
            }
            return new NiporlanItem(dataCell);
        }
        HashMap page = null;
        int index = dataCell.tableOffset;
        if (index < 0) {
            index = -index;
        }
        page = this.dataCellCache[index % NiporlanItemFactory.PAGE_COUNT];
        DataCell oldDataCell = null;
        final String mapKey = dataCell.toString();
        oldDataCell = page.get(mapKey);
        if (oldDataCell == null) {
            synchronized (page) {
                if (this.logger != null) {
                    this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "未命中 " + mapKey);
                }
                oldDataCell = page.get(mapKey);
                if (oldDataCell == null) {
                    if (this.logger != null) {
                        this.logger.println(String.valueOf(Thread.currentThread().getName()) + "\t" + "连续未命中 " + mapKey);
                    }
                    final DataCell newDataCell = new DataCell();
                    newDataCell.assign(dataCell);
                    page.put(mapKey, newDataCell);
                }
                monitorexit(page);
            }
        }
        if (oldDataCell != null && oldDataCell.statItem == dataCell.statItem && oldDataCell.offset == dataCell.offset) {
            return new NiporlanItem(oldDataCell);
        }
        return new NiporlanItem(dataCell);
    }
    
    public NiporlanItem getNiporlanItem(final Object obj) {
        return new NiporlanItem(obj);
    }
    
    public String getProductCountDesc() {
        return "Factory NO." + this.id;
    }
    
    public void destroy() {
        if (!NiporlanItemFactory.useGlobalFactory && this != NiporlanItemFactory.globalFactory) {
            this.FUNC_NIS.clear();
        }
    }
}
